package com.unleashedsoftware.sdk.example;

import com.google.inject.*;
import com.unleashedsoftware.sdk.model.*;
import com.unleashedsoftware.sdk.network.*;
import com.unleashedsoftware.sdk.security.*;
import org.codehaus.jackson.map.*;
import java.io.*;

import static java.lang.System.out;


public class SalesOrderGetById {
    public UnleashedApiCredentials credentials;

    private UnleashedApiAuthSignatureCalculator calculator;
    private RestClient client;
    private RestClientReadOnlyRequest request;
    private ObjectMapper serialiser;

    @Inject
    public SalesOrderGetById(UnleashedApiAuthSignatureCalculator calculator, RestClient client) {
        this.calculator = calculator;
        this.client = client;
        this.serialiser = new ObjectMapper();
        this.serialiser.disable(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES);
    }

    public void getById(String id) throws Exception {
        init(id);
        try {
            tryGet();
            handleSuccessGet();
        } catch (GetException error) {
            handleNotSuccessGet(error);
        } catch (Exception error) {
            handleUnexpectedError(error);
        }
    }

    private void init(String id) throws Exception {
        String url = String.format("https://api.unleashedsoftware.com/SalesOrders/%s", id);
        String query = "";
        calculateSignatureUsingQueryString(query);
        createRequest(url, query);
    }

    private void calculateSignatureUsingQueryString(String query) throws Exception {
        credentials.hash = query;
        calculator.calculate(credentials);
    }

    private void createRequest(String url, String query) throws Exception {
        request = new RestClientReadOnlyRequest();
        request.requestUrl = query.length() > 0 ? String.format("%s?%s", url, query) : url;
        request.headers.addHeader("User-Agent", "Unleashed");
        request.headers.addHeader("Content-Type", "application/json");
        request.headers.addHeader("Content-Length", Integer.toString(0));
        request.headers.addHeader("Accept", "application/json");
        request.headers.addHeader("api-auth-id", credentials.authId);
        request.headers.addHeader("api-auth-signature", credentials.authSignature);
        request.connectionTimeoutInMilliseconds = 60000;
    }

    private void tryGet() throws IOException {
        client.get(request);
        if(request.hasError())
            throw new GetException();
    }

    private void handleSuccessGet() throws IOException {
        ApiSalesOrder salesOrder = serialiser.readValue(request.requestResult, ApiSalesOrder.class);
        out.println(String.format("Id:%s, OrderNumber:%s, OrderDate:%s", salesOrder.id, salesOrder.orderNumber, salesOrder.orderDate));
    }

    private void handleNotSuccessGet(GetException error) throws IOException {
        ApiRequestError requestError = serialiser.readValue(request.requestResult, ApiRequestError.class);
        System.out.println(String.format("Description:%s", requestError.description));
        System.out.println(String.format("DebugInformation:%s", requestError.debugInformation));
    }

    private void handleUnexpectedError(Exception error) {
        throw new UnsupportedOperationException();
    }

    public class GetException extends RuntimeException {
    }
}
