package com.unleashedsoftware.sdk.network;

import java.io.*;
import java.net.*;
import java.util.*;

public abstract class RestClientConnection {
    public HttpURLConnection httpURLConnection;

    public RestClientConnection(RestClientConnectionConfiguration configuration) throws IOException {
        if(configuration.requestMethod == RestClientRequestMethod.unkown)
            throw new UnknownRequestMethodNotSupportedException();

        createAndConfigureConnection(configuration);
    }

    private void createAndConfigureConnection(RestClientConnectionConfiguration configuration) throws IOException {
        httpURLConnection = (HttpURLConnection)configuration.requestUrl.openConnection();

        httpURLConnection.setUseCaches(useCaches());
        httpURLConnection.setDoInput(isReadable());
        httpURLConnection.setDoOutput(isWritable());
        httpURLConnection.setRequestMethod(getRequestMethod(configuration));
        httpURLConnection.setConnectTimeout(configuration.connectionTimeoutInMilliseconds);

        for(int index = 0; index < configuration.headers.getSize(); index++) {
            AbstractMap.SimpleEntry<String, String> header = configuration.headers.getHeader(index);
            httpURLConnection.setRequestProperty(header.getKey(), header.getValue());
        }
    }

    private String getRequestMethod(RestClientConnectionConfiguration configuration) {
        return configuration.requestMethod.name().toUpperCase();
    }

    public InputStream getInputStream() throws IOException {
        if(getResponseCode() < 400)
            return httpURLConnection.getInputStream();
        return httpURLConnection.getErrorStream();

    }

    public OutputStream getOutputStream() throws IOException {
        return httpURLConnection.getOutputStream();
    }

    public int getResponseCode() throws IOException {
        if(isProxyConnectionExists())
            return httpURLConnection.getResponseCode();
        return 0;
    }

    public void connect() throws IOException {
        httpURLConnection.connect();
    }

    public void disconnect() {
        if(isProxyConnectionExists())
            httpURLConnection.disconnect();
    }

    private boolean isProxyConnectionExists() {
        return httpURLConnection != null;
    }

    protected boolean useCaches() {
        return false;
    }

    protected boolean isReadable() {
        return false;
    }

    protected boolean isWritable() {
        return false;
    }

    public static class UnknownRequestMethodNotSupportedException extends RuntimeException {
    }
}