package com.unleashedsoftware.sdk.network;

import org.junit.*;
import java.net.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

public class RestClientConnectionConfigurationCreatorTests {
    @Test
    public void whenCreate() throws Exception {
        // arrange
        RestClientRequestMethod requestMethod = RestClientRequestMethod.get;
        RestClientRequest request = createRequest();
        RestClientConnectionConfigurationCreator creator = new RestClientConnectionConfigurationCreator();

        // act
        RestClientConnectionConfiguration actualConfiguration = creator.create(requestMethod, request);

        // assert
        RestClientConnectionConfiguration expectedConfiguration = createConfiguration();
        assertThat(actualConfiguration.requestUrl.toString(), equalTo(expectedConfiguration.requestUrl.toString()));
        assertThat(actualConfiguration.requestMethod, equalTo(expectedConfiguration.requestMethod));
        assertThat(actualConfiguration.headers.getSize(), equalTo(expectedConfiguration.headers.getSize()));
        for(int index = 0; index < actualConfiguration.headers.getSize(); index++)
        {
            assertThat(actualConfiguration.headers.getHeader(index).getKey(), equalTo(expectedConfiguration.headers.getHeader(index).getKey()));
            assertThat(actualConfiguration.headers.getHeader(index).getValue(), equalTo(expectedConfiguration.headers.getHeader(index).getValue()));
        }
        assertThat(actualConfiguration.connectionTimeoutInMilliseconds, equalTo(expectedConfiguration.connectionTimeoutInMilliseconds));
    }

    private RestClientReadOnlyRequest createRequest() {
        RestClientReadOnlyRequest request = new RestClientReadOnlyRequest();
        request.requestUrl = "https://acme.org";
        request.headers.addHeader("field1","value1");
        request.headers.addHeader("field2","value2");
        request.headers.addHeader("field3","value3");
        request.connectionTimeoutInMilliseconds = 60000;
        return request;
    }

    private RestClientConnectionConfiguration createConfiguration() throws MalformedURLException {
        RestClientConnectionConfiguration configuration = new RestClientConnectionConfiguration();
        configuration.requestUrl = new URL("https://acme.org");
        configuration.requestMethod = RestClientRequestMethod.get;
        configuration.headers.addHeader("field1","value1");
        configuration.headers.addHeader("field2","value2");
        configuration.headers.addHeader("field3","value3");
        configuration.connectionTimeoutInMilliseconds = 60000;
        return configuration;
    }
}
