package com.unleashedsoftware.sdk.network;

import org.junit.*;
import java.net.*;

import static com.unleashedsoftware.sdk.network.RestClientConnection.*;
import static com.unleashedsoftware.sdk.network.RestClientReadWriteConnection.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

public class RestClientReadWriteConnectionTests {
    @Test(expected = GetRequestMethodNotSupportedException.class)
    public void givenAConfigurationWithGetRequestMethod_whenCreate_thenGetRequestMethodNotSupportedExceptionShouldBeThrown() throws Exception {
        RestClientConnectionConfiguration configuration = new RestClientConnectionConfiguration();
        configuration.requestMethod = RestClientRequestMethod.get;
        create(configuration);
    }

    @Test(expected = DeleteRequestMethodNotSupportedException.class)
    public void givenAConfigurationWithDeleteRequestMethod_whenCreate_thenDeleteRequestMethodNotSupportedExceptionShouldBeThrown() throws Exception {
        RestClientConnectionConfiguration configuration = new RestClientConnectionConfiguration();
        configuration.requestMethod = RestClientRequestMethod.delete;
        create(configuration);
    }

    @Test(expected = UnknownRequestMethodNotSupportedException.class)
    public void givenAConfigurationWithUnknownRequestMethod_whenCreate_thenUnknownRequestMethodNotSupportedExceptionShouldBeThrown() throws Exception {
        RestClientConnectionConfiguration configuration = new RestClientConnectionConfiguration();
        configuration.requestMethod = RestClientRequestMethod.unkown;
        create(configuration);
    }

    @Test
    public void givenAValidConfiguration_whenCreate_thenHttpURLConnectionShouldBeSetProperly() throws Exception {
        RestClientConnectionConfiguration configuration = new RestClientConnectionConfiguration();
        configuration.requestUrl = new URL("https://acme.org");
        configuration.requestMethod = RestClientRequestMethod.post;
        configuration.headers.addHeader("field1","value1");
        configuration.headers.addHeader("field2","value2");
        configuration.headers.addHeader("field3","value3");
        configuration.connectionTimeoutInMilliseconds = 60000;

        RestClientReadWriteConnection connection = RestClientReadWriteConnection.create(configuration);

        assertThat(connection.httpURLConnection, not(nullValue()));
        assertThat(connection.httpURLConnection.getURL(), equalTo(configuration.requestUrl));
        assertThat(connection.httpURLConnection.getRequestMethod(), equalTo(configuration.requestMethod.name().toUpperCase()));
        assertThat(connection.httpURLConnection.getRequestProperties().size(), equalTo(configuration.headers.getSize()));
        for(int index = 0; index < configuration.headers.getSize(); index++) {
            String propertyValue = connection.httpURLConnection.getRequestProperty(configuration.headers.getHeader(index).getKey());
            assertThat(propertyValue, not(nullValue()));
            assertThat(propertyValue, equalTo(configuration.headers.getHeader(index).getValue()));
        }
        assertThat(connection.httpURLConnection.getConnectTimeout(), equalTo(configuration.connectionTimeoutInMilliseconds));
        assertThat(connection.httpURLConnection.getUseCaches(), equalTo(false));
        assertThat(connection.httpURLConnection.getDoInput(), equalTo(true));
        assertThat(connection.httpURLConnection.getDoOutput(), equalTo(true));
    }
}
